/*
 * Copyright (c) 2010 Mysema Ltd.
 * All rights reserved.
 * 
 */
package com.mysema.commons.lang;

import java.io.Closeable;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Adapter implementation for Iterator and CloseableIterator instances
 * 
 * @author sasa
 * 
 */
public class IteratorAdapter<T> implements CloseableIterator<T> {

    private final Iterator<T> iter;

    public IteratorAdapter(Iterator<T> iter) {
        this.iter = iter;
    }

    public static <T> List<T> asList(Iterator<T> iter) {
        List<T> list = new ArrayList<T>();
        try {
            while (iter.hasNext()) {
                list.add(iter.next());
            }
        } finally {
            if (iter instanceof Closeable) {
                try {
                    ((Closeable) iter).close();
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
            }
        }
        return list;
    }

    public List<T> asList() {
        return asList(iter);
    }

    public void close() throws IOException {
        if (iter instanceof Closeable) {
            ((Closeable) iter).close();
        }
    }

    public boolean hasNext() {
        return iter.hasNext();
    }

    public T next() {
        return iter.next();
    }

    public void remove() {
        iter.remove();
    }
}
